/* 
** This is a series of C code tidbits for Microsoft C++ 
** These tidbits illustrate  how to enable and disable some of the advanced features
** of the Intellicon(2,4,8 and FLex8) serial communications card.
** The features currently supported and shown here are support
** for Multidropped RS-485 transmitters and 1/2 Duplex RS-485.
**
** FOUR WIRE Connection
**
** For a 4 wire RS-485 connection the IOCTL_SERIAL_RS485_MULTIDROP_ENABLE and
** IOCTL_SERIAL_RS485_MULTIDROP_ENABLE would need to be employed. This will
** use the RTS signal to enable/disable (tri-state) the RS-485 transmitter. When
** data is to be transmitted the RS-485 transmitter is enabled and it is disabled
** when there is no data to be transmitted.
**
** TWO WIRE (1/2) Duplex Connection
**
** For a 2 wire 1/2 duplex system, both IOCTL's need to be used. The 
** IOCTL_SERIAL_RS485_HALFDUPLEX_ENABLE	and IOCTL_SERIAL_RS485_HALFDUPLEX_ENABLE	
** will discard received characaters while the Intellicon is transmitting. And it will
** accept characters when it is not transmitting. The MULTIDROP IOCT is required
** to enable and disable the transmitter. In 1/2 duplex RS-485, there are at least two
** transmitters on the bus, the master and the slave(s) transmitters. 
**
*/


#define IOCTL_SERIAL_RS485_MULTIDROP_ENABLE	CTL_CODE(FILE_DEVICE_SERIAL_PORT,0x807,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IOCTL_SERIAL_RS485_MULTIDROP_DISABLE	CTL_CODE(FILE_DEVICE_SERIAL_PORT,0x808,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IOCTL_SERIAL_RS485_HALFDUPLEX_ENABLE	CTL_CODE(FILE_DEVICE_SERIAL_PORT,0x80B,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define IOCTL_SERIAL_RS485_HALFDUPLEX_DISABLE	CTL_CODE(FILE_DEVICE_SERIAL_PORT,0x80C,METHOD_BUFFERED,FILE_ANY_ACCESS)


   if ((  hDeviceHandle =
      CreateFile( (LPCTSTR)csPortName, GENERIC_READ | GENERIC_WRITE,
                  0,                    // exclusive access
                  NULL,                 // no security attrs
                  OPEN_EXISTING,
                  FILE_ATTRIBUTE_NORMAL | 
                  FILE_FLAG_OVERLAPPED, // overlapped I/O
                  NULL )) == (HANDLE) -1 )
   {
		AfxMessageBox( "The port could not be opened" );
		return;
   }
	
	DWORD BytesReturned;	

	if( m_MultiDropCheckBox.GetCheck() )
	{
		if( !DeviceIoControl(
  				hDeviceHandle,	// handle to device of interest
				IOCTL_SERIAL_RS485_MULTIDROP_ENABLE,	// control code of operation to perform
				NULL,	// pointer to buffer to supply input data
				0,	// size of input buffer
				NULL,	// pointer to buffer to receive output data
				0,	// size of output buffer
				&BytesReturned,	// pointer to variable to receive output byte count
				NULL 	// pointer to overlapped structure for asynchronous operation
		) )
		{
			int error = GetLastError();
			AfxMessageBox("IOCTL_SERIAL_RS485_MULTIDROP_ENABLE failed");	
		}
	}
	else
	{
		if( !DeviceIoControl(
  				hDeviceHandle,	// handle to device of interest
				IOCTL_SERIAL_RS485_MULTIDROP_DISABLE,	// control code of operation to perform
				NULL,	// pointer to buffer to supply input data
				0,	// size of input buffer
				NULL,	// pointer to buffer to receive output data
				0,	// size of output buffer
				&BytesReturned,	// pointer to variable to receive output byte count
				NULL 	// pointer to overlapped structure for asynchronous operation
		) )
		{
			int error = GetLastError();
			AfxMessageBox("IOCTL_SERIAL_RS485_MULTIDROP_ENABLE failed");	
		}
	}
	
	if( m_pHalfDuplexCheckBox->GetCheck() )
	{
		if( !DeviceIoControl(
  				hDeviceHandle,	// handle to device of interest
				IOCTL_SERIAL_RS485_HALFDUPLEX_ENABLE,	// control code of operation to perform
				NULL,	// pointer to buffer to supply input data
				0,	// size of input buffer
				NULL,	// pointer to buffer to receive output data
				0,	// size of output buffer
				&BytesReturned,	// pointer to variable to receive output byte count
				NULL 	// pointer to overlapped structure for asynchronous operation
		) )
		{
			int error = GetLastError();
			AfxMessageBox("IOCTL_SERIAL_RS485_HALFDUPLEX_ENABLE failed");	
		}
	}
	else
	{
		if( !DeviceIoControl(
  				hDeviceHandle,	// handle to device of interest
				IOCTL_SERIAL_RS485_HALFDUPLEX_DISABLE,	// control code of operation to perform
				NULL,	// pointer to buffer to supply input data
				0,	// size of input buffer
				NULL,	// pointer to buffer to receive output data
				0,	// size of output buffer
				&BytesReturned,	// pointer to variable to receive output byte count
				NULL 	// pointer to overlapped structure for asynchronous operation
		) )
		{
			int error = GetLastError();
			AfxMessageBox("IOCTL_SERIAL_RS485_HALFDUPLEX_DISABLE failed");	
		}
	}

/* End of winnt_tidbit.c */
